library angular2.src.change_detection.pipes.pipe_registry;

import "package:angular2/src/facade/collection.dart" show List, ListWrapper;
import "package:angular2/src/facade/lang.dart"
    show isBlank, isPresent, BaseException;
import "pipe.dart" show Pipe, PipeFactory;
import "package:angular2/src/di/decorators.dart" show Injectable;
import "../change_detector_ref.dart" show ChangeDetectorRef;

@Injectable()
class PipeRegistry {
  var config;
  PipeRegistry(this.config) {}
  Pipe get(String type, obj, [ChangeDetectorRef cdRef, Pipe existingPipe]) {
    if (isPresent(existingPipe) &&
        existingPipe.supports(obj)) return existingPipe;
    if (isPresent(existingPipe)) existingPipe.onDestroy();
    var factories = this._getListOfFactories(type, obj);
    var factory = this._getMatchingFactory(factories, type, obj);
    return factory.create(cdRef);
  }
  List<PipeFactory> _getListOfFactories(String type, dynamic obj) {
    var listOfFactories = this.config[type];
    if (isBlank(listOfFactories)) {
      throw new BaseException(
          '''Cannot find \'${ type}\' pipe supporting object \'${ obj}\'''');
    }
    return listOfFactories;
  }
  PipeFactory _getMatchingFactory(
      List<PipeFactory> listOfFactories, String type, dynamic obj) {
    var matchingFactory = ListWrapper.find(
        listOfFactories, (pipeFactory) => pipeFactory.supports(obj));
    if (isBlank(matchingFactory)) {
      throw new BaseException(
          '''Cannot find \'${ type}\' pipe supporting object \'${ obj}\'''');
    }
    return matchingFactory;
  }
}
