library angular2.src.core.compiler.element_injector;

import "package:angular2/src/facade/lang.dart"
    show isPresent, isBlank, Type, BaseException, stringify, StringWrapper;
import "package:angular2/src/facade/async.dart"
    show EventEmitter, ObservableWrapper;
import "package:angular2/src/facade/collection.dart"
    show List, ListWrapper, MapWrapper, StringMapWrapper;
import "package:angular2/di.dart"
    show
        Injector,
        Key,
        Dependency,
        bind,
        Binding,
        ResolvedBinding,
        NoBindingError,
        AbstractBindingError,
        CyclicDependencyError,
        resolveForwardRef,
        resolveBindings;
import "package:angular2/src/core/annotations_impl/visibility.dart"
    show Visibility, self;
import "package:angular2/src/core/annotations_impl/di.dart"
    show Attribute, Query;
import "view.dart" as viewModule;
import "view_manager.dart" as avmModule;
import "view_container_ref.dart" show ViewContainerRef;
import "element_ref.dart" show ElementRef;
import "view_ref.dart" show ProtoViewRef, ViewRef;
import "package:angular2/src/core/annotations_impl/annotations.dart"
    show
        Directive,
        Component,
        onChange,
        onDestroy,
        onCheck,
        onInit,
        onAllChangesDone;
import "directive_lifecycle_reflector.dart" show hasLifecycleHook;
import "package:angular2/change_detection.dart"
    show ChangeDetector, ChangeDetectorRef;
import "query_list.dart" show QueryList;
import "package:angular2/src/reflection/reflection.dart" show reflector;
import "package:angular2/src/render/api.dart" show DirectiveMetadata;

// Threshold for the dynamic version
var _MAX_DIRECTIVE_CONSTRUCTION_COUNTER = 10;
const _undefined = const Object();
var _staticKeys;
class StaticKeys {
  num viewManagerId;
  num protoViewId;
  num viewContainerId;
  num changeDetectorRefId;
  num elementRefId;
  StaticKeys() {
    // TODO: vsavkin Key.annotate(Key.get(AppView), 'static')
    this.viewManagerId = Key.get(avmModule.AppViewManager).id;
    this.protoViewId = Key.get(ProtoViewRef).id;
    this.viewContainerId = Key.get(ViewContainerRef).id;
    this.changeDetectorRefId = Key.get(ChangeDetectorRef).id;
    this.elementRefId = Key.get(ElementRef).id;
  }
  static StaticKeys instance() {
    if (isBlank(_staticKeys)) _staticKeys = new StaticKeys();
    return _staticKeys;
  }
}
class TreeNode<T extends TreeNode<dynamic>> {
  T _parent;
  T _head = null;
  T _tail = null;
  T _next = null;
  TreeNode(T parent) {
    if (isPresent(parent)) parent.addChild(this);
  }
  /**
   * Adds a child to the parent node. The child MUST NOT be a part of a tree.
   */
  void addChild(T child) {
    if (isPresent(this._tail)) {
      this._tail._next = child;
      this._tail = child;
    } else {
      this._tail = this._head = child;
    }
    child._next = null;
    child._parent = this;
  }
  /**
   * Adds a child to the parent node after a given sibling.
   * The child MUST NOT be a part of a tree and the sibling must be present.
   */
  void addChildAfter(T child, T prevSibling) {
    if (isBlank(prevSibling)) {
      var prevHead = this._head;
      this._head = child;
      child._next = prevHead;
      if (isBlank(this._tail)) this._tail = child;
    } else if (isBlank(prevSibling._next)) {
      this.addChild(child);
      return;
    } else {
      child._next = prevSibling._next;
      prevSibling._next = child;
    }
    child._parent = this;
  }
  /**
   * Detaches a node from the parent's tree.
   */
  void remove() {
    if (isBlank(this.parent)) return;
    var nextSibling = this._next;
    var prevSibling = this._findPrev();
    if (isBlank(prevSibling)) {
      this.parent._head = this._next;
    } else {
      prevSibling._next = this._next;
    }
    if (isBlank(nextSibling)) {
      this._parent._tail = prevSibling;
    }
    this._parent = null;
    this._next = null;
  }
  /**
   * Finds a previous sibling or returns null if first child.
   * Assumes the node has a parent.
   * TODO(rado): replace with DoublyLinkedList to avoid O(n) here.
   */
  _findPrev() {
    var node = this.parent._head;
    if (node == this) return null;
    while (!identical(node._next, this)) node = node._next;
    return node;
  }
  get parent {
    return this._parent;
  }
  // TODO(rado): replace with a function call, does too much work for a getter.
  List<T> get children {
    var res = [];
    var child = this._head;
    while (child != null) {
      res.add(child);
      child = child._next;
    }
    return res;
  }
}
class DependencyWithVisibility extends Dependency {
  Visibility visibility;
  DependencyWithVisibility(Key key, bool asPromise, bool lazy, bool optional,
      List<dynamic> properties, this.visibility)
      : super(key, asPromise, lazy, optional, properties) {
    /* super call moved to initializer */;
  }
  static Dependency createFrom(Dependency d) {
    return new DependencyWithVisibility(d.key, d.asPromise, d.lazy, d.optional,
        d.properties, DependencyWithVisibility._visibility(d.properties));
  }
  static Visibility _visibility(properties) {
    if (properties.length == 0) return self;
    var p = ListWrapper.find(properties, (p) => p is Visibility);
    return isPresent(p) ? p : self;
  }
}
class DirectiveDependency extends DependencyWithVisibility {
  String attributeName;
  Query queryDecorator;
  DirectiveDependency(Key key, bool asPromise, bool lazy, bool optional,
      List<dynamic> properties, Visibility visibility, this.attributeName,
      this.queryDecorator)
      : super(key, asPromise, lazy, optional, properties, visibility) {
    /* super call moved to initializer */;
    this._verify();
  }
  void _verify() {
    var count = 0;
    if (isPresent(this.queryDecorator)) count++;
    if (isPresent(this.attributeName)) count++;
    if (count > 1) throw new BaseException(
        "A directive injectable can contain only one of the following @Attribute or @Query.");
  }
  static Dependency createFrom(Dependency d) {
    return new DirectiveDependency(d.key, d.asPromise, d.lazy, d.optional,
        d.properties, DependencyWithVisibility._visibility(d.properties),
        DirectiveDependency._attributeName(d.properties),
        DirectiveDependency._query(d.properties));
  }
  static String _attributeName(properties) {
    var p = ListWrapper.find(properties, (p) => p is Attribute);
    return isPresent(p) ? p.attributeName : null;
  }
  static _query(properties) {
    return ListWrapper.find(properties, (p) => p is Query);
  }
}
class DirectiveBinding extends ResolvedBinding {
  List<ResolvedBinding> resolvedAppInjectables;
  List<ResolvedBinding> resolvedHostInjectables;
  List<ResolvedBinding> resolvedViewInjectables;
  DirectiveMetadata metadata;
  DirectiveBinding(Key key, Function factory, List<Dependency> dependencies,
      bool providedAsPromise, this.resolvedAppInjectables,
      this.resolvedHostInjectables, this.resolvedViewInjectables, this.metadata)
      : super(key, factory, dependencies, providedAsPromise) {
    /* super call moved to initializer */;
  }
  bool get callOnDestroy {
    return this.metadata.callOnDestroy;
  }
  bool get callOnChange {
    return this.metadata.callOnChange;
  }
  bool get callOnAllChangesDone {
    return this.metadata.callOnAllChangesDone;
  }
  String get displayName {
    return this.key.displayName;
  }
  List<String> get eventEmitters {
    return isPresent(this.metadata) && isPresent(this.metadata.events)
        ? this.metadata.events
        : [];
  }
  Map<String, String> get hostActions {
    return isPresent(this.metadata) && isPresent(this.metadata.hostActions)
        ? this.metadata.hostActions
        : new Map();
  }
  get changeDetection {
    return this.metadata.changeDetection;
  }
  static DirectiveBinding createFromBinding(Binding binding, Directive ann) {
    if (isBlank(ann)) {
      ann = new Directive();
    }
    var rb = binding.resolve();
    var deps = ListWrapper.map(rb.dependencies, DirectiveDependency.createFrom);
    var resolvedAppInjectables = ann is Component && isPresent(ann.appInjector)
        ? Injector.resolve(ann.appInjector)
        : [];
    var resolvedHostInjectables =
        isPresent(ann.hostInjector) ? resolveBindings(ann.hostInjector) : [];
    var resolvedViewInjectables = ann is Component &&
        isPresent(ann.viewInjector) ? resolveBindings(ann.viewInjector) : [];
    var metadata = DirectiveMetadata.create(
        id: stringify(rb.key.token),
        type: ann is Component
            ? DirectiveMetadata.COMPONENT_TYPE
            : DirectiveMetadata.DIRECTIVE_TYPE,
        selector: ann.selector,
        compileChildren: ann.compileChildren,
        events: ann.events,
        host: isPresent(ann.host)
            ? MapWrapper.createFromStringMap(ann.host)
            : null,
        properties: ann.properties,
        readAttributes: DirectiveBinding._readAttributes(deps),
        callOnDestroy: hasLifecycleHook(onDestroy, rb.key.token, ann),
        callOnChange: hasLifecycleHook(onChange, rb.key.token, ann),
        callOnCheck: hasLifecycleHook(onCheck, rb.key.token, ann),
        callOnInit: hasLifecycleHook(onInit, rb.key.token, ann),
        callOnAllChangesDone: hasLifecycleHook(
            onAllChangesDone, rb.key.token, ann),
        changeDetection: ann is Component ? ann.changeDetection : null,
        exportAs: ann.exportAs);
    return new DirectiveBinding(rb.key, rb.factory, deps, rb.providedAsPromise,
        resolvedAppInjectables, resolvedHostInjectables,
        resolvedViewInjectables, metadata);
  }
  static _readAttributes(deps) {
    var readAttributes = [];
    ListWrapper.forEach(deps, (dep) {
      if (isPresent(dep.attributeName)) {
        readAttributes.add(dep.attributeName);
      }
    });
    return readAttributes;
  }
  static DirectiveBinding createFromType(Type type, Directive annotation) {
    var binding = new Binding(type, toClass: type);
    return DirectiveBinding.createFromBinding(binding, annotation);
  }
}
// TODO(rado): benchmark and consider rolling in as ElementInjector fields.
class PreBuiltObjects {
  avmModule.AppViewManager viewManager;
  viewModule.AppView view;
  viewModule.AppProtoView protoView;
  PreBuiltObjects(this.viewManager, this.view, this.protoView) {}
}
class EventEmitterAccessor {
  String eventName;
  Function getter;
  EventEmitterAccessor(this.eventName, this.getter) {}
  subscribe(viewModule.AppView view, num boundElementIndex, Object directive) {
    var eventEmitter = this.getter(directive);
    return ObservableWrapper.subscribe(eventEmitter, (eventObj) =>
        view.triggerEventHandlers(this.eventName, eventObj, boundElementIndex));
  }
}
class HostActionAccessor {
  String methodName;
  Function getter;
  HostActionAccessor(this.methodName, this.getter) {}
  subscribe(viewModule.AppView view, num boundElementIndex, Object directive) {
    var eventEmitter = this.getter(directive);
    return ObservableWrapper.subscribe(eventEmitter, (actionArgs) => view
        .invokeElementMethod(boundElementIndex, this.methodName, actionArgs));
  }
}
const LIGHT_DOM = 1;
const SHADOW_DOM = 2;
const LIGHT_DOM_AND_SHADOW_DOM = 3;
class BindingData {
  ResolvedBinding binding;
  num visibility;
  BindingData(this.binding, this.visibility) {}
  getKeyId() {
    return this.binding.key.id;
  }
  createEventEmitterAccessors() {
    if (!(this.binding is DirectiveBinding)) return [];
    var db = (this.binding as DirectiveBinding);
    return ListWrapper.map(db.eventEmitters, (eventConfig) {
      var fieldName;
      var eventName;
      var colonIdx = eventConfig.indexOf(":");
      if (colonIdx > -1) {
        // long format: 'fieldName: eventName'
        fieldName = StringWrapper.substring(eventConfig, 0, colonIdx).trim();
        eventName = StringWrapper.substring(eventConfig, colonIdx + 1).trim();
      } else {
        // short format: 'name' when fieldName and eventName are the same
        fieldName = eventName = eventConfig;
      }
      return new EventEmitterAccessor(eventName, reflector.getter(fieldName));
    });
  }
  createHostActionAccessors() {
    if (!(this.binding is DirectiveBinding)) return [];
    var res = [];
    var db = (this.binding as DirectiveBinding);
    MapWrapper.forEach(db.hostActions, (actionExpression, actionName) {
      res.add(new HostActionAccessor(
          actionExpression, reflector.getter(actionName)));
    });
    return res;
  }
}
/**

Difference between di.Injector and ElementInjector

di.Injector:
 - imperative based (can create child injectors imperativly)
 - Lazy loading of code
 - Component/App Level services which are usually not DOM Related.


ElementInjector:
  - ProtoBased (Injector structure fixed at compile time)
  - understands @Ancestor, @Parent, @Child, @Descendent
  - Fast
  - Query mechanism for children
  - 1:1 to DOM structure.

 PERF BENCHMARK:
http://www.williambrownstreet.net/blog/2014/04/faster-angularjs-rendering-angularjs-and-reactjs/
 */
class ProtoElementInjector {
  ProtoElementInjector parent;
  int index;
  num distanceToParent;
  bool _firstBindingIsComponent;
  Map<String, num> directiveVariableBindings;
  viewModule.AppView view;
  Map<String, String> attributes;
  List<List<EventEmitterAccessor>> eventEmitterAccessors;
  List<List<HostActionAccessor>> hostActionAccessors;
  _ProtoElementInjectorStrategy _strategy;
  static create(ProtoElementInjector parent, num index,
      List<ResolvedBinding> bindings, bool firstBindingIsComponent,
      num distanceToParent, Map<String, num> directiveVariableBindings) {
    var bd = [];
    ProtoElementInjector._createDirectiveBindingData(
        bindings, bd, firstBindingIsComponent);
    if (firstBindingIsComponent) {
      ProtoElementInjector._createViewInjectorBindingData(bindings, bd);
    }
    ProtoElementInjector._createHostInjectorBindingData(
        bindings, bd, firstBindingIsComponent);
    return new ProtoElementInjector(parent, index, bd, distanceToParent,
        firstBindingIsComponent, directiveVariableBindings);
  }
  static _createDirectiveBindingData(List<ResolvedBinding> dirBindings,
      List<BindingData> bd, bool firstBindingIsComponent) {
    ListWrapper.forEach(dirBindings, (dirBinding) {
      bd.add(ProtoElementInjector._createBindingData(
          firstBindingIsComponent, dirBinding, dirBindings, dirBinding));
    });
  }
  static _createHostInjectorBindingData(List<ResolvedBinding> dirBindings,
      List<BindingData> bd, bool firstBindingIsComponent) {
    ListWrapper.forEach(dirBindings, (dirBinding) {
      ListWrapper.forEach(dirBinding.resolvedHostInjectables, (b) {
        bd.add(ProtoElementInjector._createBindingData(firstBindingIsComponent,
            dirBinding, dirBindings, ProtoElementInjector._createBinding(b)));
      });
    });
  }
  static _createBindingData(
      firstBindingIsComponent, dirBinding, dirBindings, binding) {
    var isComponent =
        firstBindingIsComponent && identical(dirBindings[0], dirBinding);
    return new BindingData(
        binding, isComponent ? LIGHT_DOM_AND_SHADOW_DOM : LIGHT_DOM);
  }
  static _createViewInjectorBindingData(
      List<ResolvedBinding> bindings, List<BindingData> bd) {
    var db = (bindings[0] as DirectiveBinding);
    ListWrapper.forEach(db.resolvedViewInjectables, (b) => bd.add(
        new BindingData(ProtoElementInjector._createBinding(b), SHADOW_DOM)));
  }
  static _createBinding(ResolvedBinding b) {
    var deps = ListWrapper.map(
        b.dependencies, (d) => DependencyWithVisibility.createFrom(d));
    return new ResolvedBinding(b.key, b.factory, deps, b.providedAsPromise);
  }
  ProtoElementInjector(this.parent, this.index, List<BindingData> bd,
      this.distanceToParent, this._firstBindingIsComponent,
      this.directiveVariableBindings) {
    var length = bd.length;
    this.eventEmitterAccessors = ListWrapper.createFixedSize(length);
    this.hostActionAccessors = ListWrapper.createFixedSize(length);
    this._strategy = length > _MAX_DIRECTIVE_CONSTRUCTION_COUNTER
        ? new _ProtoElementInjectorDynamicStrategy(this, bd)
        : new _ProtoElementInjectorInlineStrategy(this, bd);
  }
  ElementInjector instantiate(ElementInjector parent) {
    return new ElementInjector(this, parent);
  }
  ProtoElementInjector directParent() {
    return this.distanceToParent < 2 ? this.parent : null;
  }
  bool get hasBindings {
    return this._strategy.hasBindings();
  }
  dynamic getBindingAtIndex(num index) {
    return this._strategy.getBindingAtIndex(index);
  }
}
abstract class _ProtoElementInjectorStrategy {
  bool hasBindings();
  dynamic getBindingAtIndex(num index);
  _ElementInjectorStrategy createElementInjectorStrategy(ElementInjector ei);
}
/**
 * Strategy used by the `ProtoElementInjector` when the number of bindings is 10 or less.
 * In such a case, inlining fields is benefitial for performances.
 */
class _ProtoElementInjectorInlineStrategy
    implements _ProtoElementInjectorStrategy {
  // only _binding0 can contain a component
  ResolvedBinding _binding0 = null;
  ResolvedBinding _binding1 = null;
  ResolvedBinding _binding2 = null;
  ResolvedBinding _binding3 = null;
  ResolvedBinding _binding4 = null;
  ResolvedBinding _binding5 = null;
  ResolvedBinding _binding6 = null;
  ResolvedBinding _binding7 = null;
  ResolvedBinding _binding8 = null;
  ResolvedBinding _binding9 = null;
  num _keyId0 = null;
  num _keyId1 = null;
  num _keyId2 = null;
  num _keyId3 = null;
  num _keyId4 = null;
  num _keyId5 = null;
  num _keyId6 = null;
  num _keyId7 = null;
  num _keyId8 = null;
  num _keyId9 = null;
  num _visibility0 = null;
  num _visibility1 = null;
  num _visibility2 = null;
  num _visibility3 = null;
  num _visibility4 = null;
  num _visibility5 = null;
  num _visibility6 = null;
  num _visibility7 = null;
  num _visibility8 = null;
  num _visibility9 = null;
  _ProtoElementInjectorInlineStrategy(
      ProtoElementInjector protoEI, List<BindingData> bd) {
    var length = bd.length;
    if (length > 0) {
      this._binding0 = bd[0].binding;
      this._keyId0 = bd[0].getKeyId();
      this._visibility0 = bd[0].visibility;
      protoEI.eventEmitterAccessors[0] = bd[0].createEventEmitterAccessors();
      protoEI.hostActionAccessors[0] = bd[0].createHostActionAccessors();
    }
    if (length > 1) {
      this._binding1 = bd[1].binding;
      this._keyId1 = bd[1].getKeyId();
      this._visibility1 = bd[1].visibility;
      protoEI.eventEmitterAccessors[1] = bd[1].createEventEmitterAccessors();
      protoEI.hostActionAccessors[1] = bd[1].createHostActionAccessors();
    }
    if (length > 2) {
      this._binding2 = bd[2].binding;
      this._keyId2 = bd[2].getKeyId();
      this._visibility2 = bd[2].visibility;
      protoEI.eventEmitterAccessors[2] = bd[2].createEventEmitterAccessors();
      protoEI.hostActionAccessors[2] = bd[2].createHostActionAccessors();
    }
    if (length > 3) {
      this._binding3 = bd[3].binding;
      this._keyId3 = bd[3].getKeyId();
      this._visibility3 = bd[3].visibility;
      protoEI.eventEmitterAccessors[3] = bd[3].createEventEmitterAccessors();
      protoEI.hostActionAccessors[3] = bd[3].createHostActionAccessors();
    }
    if (length > 4) {
      this._binding4 = bd[4].binding;
      this._keyId4 = bd[4].getKeyId();
      this._visibility4 = bd[4].visibility;
      protoEI.eventEmitterAccessors[4] = bd[4].createEventEmitterAccessors();
      protoEI.hostActionAccessors[4] = bd[4].createHostActionAccessors();
    }
    if (length > 5) {
      this._binding5 = bd[5].binding;
      this._keyId5 = bd[5].getKeyId();
      this._visibility5 = bd[5].visibility;
      protoEI.eventEmitterAccessors[5] = bd[5].createEventEmitterAccessors();
      protoEI.hostActionAccessors[5] = bd[5].createHostActionAccessors();
    }
    if (length > 6) {
      this._binding6 = bd[6].binding;
      this._keyId6 = bd[6].getKeyId();
      this._visibility6 = bd[6].visibility;
      protoEI.eventEmitterAccessors[6] = bd[6].createEventEmitterAccessors();
      protoEI.hostActionAccessors[6] = bd[6].createHostActionAccessors();
    }
    if (length > 7) {
      this._binding7 = bd[7].binding;
      this._keyId7 = bd[7].getKeyId();
      this._visibility7 = bd[7].visibility;
      protoEI.eventEmitterAccessors[7] = bd[7].createEventEmitterAccessors();
      protoEI.hostActionAccessors[7] = bd[7].createHostActionAccessors();
    }
    if (length > 8) {
      this._binding8 = bd[8].binding;
      this._keyId8 = bd[8].getKeyId();
      this._visibility8 = bd[8].visibility;
      protoEI.eventEmitterAccessors[8] = bd[8].createEventEmitterAccessors();
      protoEI.hostActionAccessors[8] = bd[8].createHostActionAccessors();
    }
    if (length > 9) {
      this._binding9 = bd[9].binding;
      this._keyId9 = bd[9].getKeyId();
      this._visibility9 = bd[9].visibility;
      protoEI.eventEmitterAccessors[9] = bd[9].createEventEmitterAccessors();
      protoEI.hostActionAccessors[9] = bd[9].createHostActionAccessors();
    }
  }
  bool hasBindings() {
    return isPresent(this._binding0);
  }
  dynamic getBindingAtIndex(num index) {
    if (index == 0) return this._binding0;
    if (index == 1) return this._binding1;
    if (index == 2) return this._binding2;
    if (index == 3) return this._binding3;
    if (index == 4) return this._binding4;
    if (index == 5) return this._binding5;
    if (index == 6) return this._binding6;
    if (index == 7) return this._binding7;
    if (index == 8) return this._binding8;
    if (index == 9) return this._binding9;
    throw new OutOfBoundsAccess(index);
  }
  _ElementInjectorStrategy createElementInjectorStrategy(ElementInjector ei) {
    return new ElementInjectorInlineStrategy(this, ei);
  }
}
/**
 * Strategy used by the `ProtoElementInjector` when the number of bindings is more than 10.
 */
class _ProtoElementInjectorDynamicStrategy
    implements _ProtoElementInjectorStrategy {
  // only _bindings[0] can contain a component
  List<ResolvedBinding> _bindings;
  List<num> _keyIds;
  List<num> _visibilities;
  _ProtoElementInjectorDynamicStrategy(
      ProtoElementInjector protoInj, List<BindingData> bd) {
    var len = bd.length;
    this._bindings = ListWrapper.createFixedSize(len);
    this._keyIds = ListWrapper.createFixedSize(len);
    this._visibilities = ListWrapper.createFixedSize(len);
    for (var i = 0; i < len; i++) {
      this._bindings[i] = bd[i].binding;
      this._keyIds[i] = bd[i].getKeyId();
      this._visibilities[i] = bd[i].visibility;
      protoInj.eventEmitterAccessors[i] = bd[i].createEventEmitterAccessors();
      protoInj.hostActionAccessors[i] = bd[i].createHostActionAccessors();
    }
  }
  bool hasBindings() {
    return isPresent(this._bindings[0]);
  }
  dynamic getBindingAtIndex(num index) {
    if (index < 0 || index >= this._bindings.length) {
      throw new OutOfBoundsAccess(index);
    }
    return this._bindings[index];
  }
  createElementInjectorStrategy(ElementInjector ei) {
    return new ElementInjectorDynamicStrategy(this, ei);
  }
}
class ElementInjector extends TreeNode<ElementInjector> {
  ProtoElementInjector _proto;
  Injector _lightDomAppInjector = null;
  Injector _shadowDomAppInjector = null;
  ElementInjector _host;
  var _preBuiltObjects = null;
  num _constructionCounter = 0;
  // Queries are added during construction or linking with a new parent.

  // They are never removed.
  QueryRef _query0;
  QueryRef _query1;
  QueryRef _query2;
  bool hydrated;
  _ElementInjectorStrategy _strategy;
  ElementInjector(this._proto, ElementInjector parent) : super(parent) {
    /* super call moved to initializer */;
    this._strategy = _proto._strategy.createElementInjectorStrategy(this);
    this._constructionCounter = 0;
    this.hydrated = false;
    this._buildQueries();
    this._addParentQueries();
  }
  void dehydrate() {
    this.hydrated = false;
    this._host = null;
    this._preBuiltObjects = null;
    this._lightDomAppInjector = null;
    this._shadowDomAppInjector = null;
    this._strategy.callOnDestroy();
    this._strategy.clearInstances();
    this._constructionCounter = 0;
  }
  void onAllChangesDone() {
    if (isPresent(this._query0) && identical(this._query0.originator, this)) {
      this._query0.list.fireCallbacks();
    }
    if (isPresent(this._query1) && identical(this._query1.originator, this)) {
      this._query1.list.fireCallbacks();
    }
    if (isPresent(this._query2) && identical(this._query2.originator, this)) {
      this._query2.list.fireCallbacks();
    }
  }
  void hydrate(Injector injector, ElementInjector host,
      PreBuiltObjects preBuiltObjects) {
    var p = this._proto;
    this._host = host;
    this._lightDomAppInjector = injector;
    this._preBuiltObjects = preBuiltObjects;
    if (p._firstBindingIsComponent) {
      this._shadowDomAppInjector = this._createShadowDomAppInjector(
          this._strategy.getComponentBinding(), injector);
    }
    this._checkShadowDomAppInjector(this._shadowDomAppInjector);
    this._strategy.hydrate();
    this._addVarBindingsToQueries();
    this.hydrated = true;
  }
  bool hasVariableBinding(String name) {
    var vb = this._proto.directiveVariableBindings;
    return isPresent(vb) && vb.containsKey(name);
  }
  dynamic getVariableBinding(String name) {
    var index = this._proto.directiveVariableBindings[name];
    return isPresent(index)
        ? this.getDirectiveAtIndex((index as num))
        : this.getElementRef();
  }
  Injector _createShadowDomAppInjector(
      DirectiveBinding componentDirective, Injector appInjector) {
    if (!ListWrapper.isEmpty(componentDirective.resolvedAppInjectables)) {
      return appInjector
          .createChildFromResolved(componentDirective.resolvedAppInjectables);
    } else {
      return appInjector;
    }
  }
  void _checkShadowDomAppInjector(Injector shadowDomAppInjector) {
    if (this._proto._firstBindingIsComponent && isBlank(shadowDomAppInjector)) {
      throw new BaseException(
          "A shadowDomAppInjector is required as this ElementInjector contains a component");
    } else if (!this._proto._firstBindingIsComponent &&
        isPresent(shadowDomAppInjector)) {
      throw new BaseException(
          "No shadowDomAppInjector allowed as there is not component stored in this ElementInjector");
    }
  }
  dynamic get(token) {
    return this._getByKey(Key.get(token), self, false, null);
  }
  bool hasDirective(Type type) {
    return !identical(this._strategy.getObjByKeyId(
        Key.get(type).id, LIGHT_DOM_AND_SHADOW_DOM), _undefined);
  }
  List<List<EventEmitterAccessor>> getEventEmitterAccessors() {
    return this._proto.eventEmitterAccessors;
  }
  List<List<HostActionAccessor>> getHostActionAccessors() {
    return this._proto.hostActionAccessors;
  }
  Map<String, num> getDirectiveVariableBindings() {
    return this._proto.directiveVariableBindings;
  }
  dynamic getComponent() {
    return this._strategy.getComponent();
  }
  ElementRef getElementRef() {
    return this._preBuiltObjects.view.elementRefs[this._proto.index];
  }
  ViewContainerRef getViewContainerRef() {
    return new ViewContainerRef(
        this._preBuiltObjects.viewManager, this.getElementRef());
  }
  ElementInjector directParent() {
    return this._proto.distanceToParent < 2 ? this.parent : null;
  }
  bool _isComponentKey(Key key) {
    return this._strategy.isComponentKey(key);
  }
  dynamic _new(ResolvedBinding binding) {
    if (this._constructionCounter++ > this._strategy.getMaxDirectives()) {
      throw new CyclicDependencyError(binding.key);
    }
    var factory = binding.factory;
    var deps = (binding.dependencies as List<DirectiveDependency>);
    var length = deps.length;
    var d0, d1, d2, d3, d4, d5, d6, d7, d8, d9;
    try {
      d0 = length > 0 ? this._getByDependency(deps[0], binding.key) : null;
      d1 = length > 1 ? this._getByDependency(deps[1], binding.key) : null;
      d2 = length > 2 ? this._getByDependency(deps[2], binding.key) : null;
      d3 = length > 3 ? this._getByDependency(deps[3], binding.key) : null;
      d4 = length > 4 ? this._getByDependency(deps[4], binding.key) : null;
      d5 = length > 5 ? this._getByDependency(deps[5], binding.key) : null;
      d6 = length > 6 ? this._getByDependency(deps[6], binding.key) : null;
      d7 = length > 7 ? this._getByDependency(deps[7], binding.key) : null;
      d8 = length > 8 ? this._getByDependency(deps[8], binding.key) : null;
      d9 = length > 9 ? this._getByDependency(deps[9], binding.key) : null;
    } catch (e, e_stack) {
      if (e is AbstractBindingError) e.addKey(binding.key);
      rethrow;
    }
    var obj;
    switch (length) {
      case 0:
        obj = factory();
        break;
      case 1:
        obj = factory(d0);
        break;
      case 2:
        obj = factory(d0, d1);
        break;
      case 3:
        obj = factory(d0, d1, d2);
        break;
      case 4:
        obj = factory(d0, d1, d2, d3);
        break;
      case 5:
        obj = factory(d0, d1, d2, d3, d4);
        break;
      case 6:
        obj = factory(d0, d1, d2, d3, d4, d5);
        break;
      case 7:
        obj = factory(d0, d1, d2, d3, d4, d5, d6);
        break;
      case 8:
        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7);
        break;
      case 9:
        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8);
        break;
      case 10:
        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9);
        break;
    }
    this._addToQueries(obj, binding.key.token);
    return obj;
  }
  dynamic _getByDependency(DependencyWithVisibility dep, Key requestor) {
    if (!(dep is DirectiveDependency)) {
      return this._getByKey(dep.key, dep.visibility, dep.optional, requestor);
    }
    var dirDep = (dep as DirectiveDependency);
    if (isPresent(dirDep.attributeName)) return this._buildAttribute(dirDep);
    if (isPresent(dirDep.queryDecorator)) return this
        ._findQuery(dirDep.queryDecorator).list;
    if (identical(dirDep.key.id, StaticKeys.instance().changeDetectorRefId)) {
      var componentView =
          this._preBuiltObjects.view.componentChildViews[this._proto.index];
      return componentView.changeDetector.ref;
    }
    if (identical(dirDep.key.id, StaticKeys.instance().elementRefId)) {
      return this.getElementRef();
    }
    if (identical(dirDep.key.id, StaticKeys.instance().viewContainerId)) {
      return this.getViewContainerRef();
    }
    if (identical(dirDep.key.id, StaticKeys.instance().protoViewId)) {
      if (isBlank(this._preBuiltObjects.protoView)) {
        if (dirDep.optional) {
          return null;
        }
        throw new NoBindingError(dirDep.key);
      }
      return new ProtoViewRef(this._preBuiltObjects.protoView);
    }
    return this._getByKey(
        dirDep.key, dirDep.visibility, dirDep.optional, requestor);
  }
  String _buildAttribute(DirectiveDependency dep) {
    var attributes = this._proto.attributes;
    if (isPresent(attributes) && attributes.containsKey(dep.attributeName)) {
      return attributes[dep.attributeName];
    } else {
      return null;
    }
  }
  void _buildQueriesForDeps(List<DirectiveDependency> deps) {
    for (var i = 0; i < deps.length; i++) {
      var dep = deps[i];
      if (isPresent(dep.queryDecorator)) {
        this._createQueryRef(dep.queryDecorator);
      }
    }
  }
  void _addVarBindingsToQueries() {
    this._addVarBindingsToQuery(this._query0);
    this._addVarBindingsToQuery(this._query1);
    this._addVarBindingsToQuery(this._query2);
  }
  void _addVarBindingsToQuery(QueryRef queryRef) {
    if (isBlank(queryRef) || !queryRef.query.isVarBindingQuery) return;
    var vb = queryRef.query.varBindings;
    for (var i = 0; i < vb.length; ++i) {
      if (this.hasVariableBinding(vb[i])) {
        queryRef.list.add(this.getVariableBinding(vb[i]));
      }
    }
  }
  void _createQueryRef(Query query) {
    var queryList = new QueryList<dynamic>();
    if (isBlank(this._query0)) {
      this._query0 = new QueryRef(query, queryList, this);
    } else if (isBlank(this._query1)) {
      this._query1 = new QueryRef(query, queryList, this);
    } else if (isBlank(this._query2)) {
      this._query2 = new QueryRef(query, queryList, this);
    } else throw new QueryError();
  }
  void _addToQueries(obj, token) {
    if (isPresent(this._query0) &&
        (identical(this._query0.query.selector, token))) {
      this._query0.list.add(obj);
    }
    if (isPresent(this._query1) &&
        (identical(this._query1.query.selector, token))) {
      this._query1.list.add(obj);
    }
    if (isPresent(this._query2) &&
        (identical(this._query2.query.selector, token))) {
      this._query2.list.add(obj);
    }
  }
  void addDirectivesMatchingQuery(Query query, List<dynamic> list) {
    this._strategy.addDirectivesMatchingQuery(query, list);
  }
  void _buildQueries() {
    if (isPresent(this._proto)) {
      this._strategy.buildQueries();
    }
  }
  QueryRef _findQuery(query) {
    if (isPresent(this._query0) && identical(this._query0.query, query)) {
      return this._query0;
    }
    if (isPresent(this._query1) && identical(this._query1.query, query)) {
      return this._query1;
    }
    if (isPresent(this._query2) && identical(this._query2.query, query)) {
      return this._query2;
    }
    throw new BaseException('''Cannot find query for directive ${ query}.''');
  }
  bool _hasQuery(QueryRef query) {
    return this._query0 == query ||
        this._query1 == query ||
        this._query2 == query;
  }
  void link(ElementInjector parent) {
    parent.addChild(this);
    this._addParentQueries();
  }
  void linkAfter(ElementInjector parent, ElementInjector prevSibling) {
    parent.addChildAfter(this, prevSibling);
    this._addParentQueries();
  }
  void _addParentQueries() {
    if (isBlank(this.parent)) return;
    if (isPresent(this.parent._query0)) {
      this._addQueryToTree(this.parent._query0);
      if (this.hydrated) this.parent._query0.update();
    }
    if (isPresent(this.parent._query1)) {
      this._addQueryToTree(this.parent._query1);
      if (this.hydrated) this.parent._query1.update();
    }
    if (isPresent(this.parent._query2)) {
      this._addQueryToTree(this.parent._query2);
      if (this.hydrated) this.parent._query2.update();
    }
  }
  void unlink() {
    var queriesToUpdate = [];
    if (isPresent(this.parent._query0)) {
      this._pruneQueryFromTree(this.parent._query0);
      queriesToUpdate.add(this.parent._query0);
    }
    if (isPresent(this.parent._query1)) {
      this._pruneQueryFromTree(this.parent._query1);
      queriesToUpdate.add(this.parent._query1);
    }
    if (isPresent(this.parent._query2)) {
      this._pruneQueryFromTree(this.parent._query2);
      queriesToUpdate.add(this.parent._query2);
    }
    this.remove();
    ListWrapper.forEach(queriesToUpdate, (q) => q.update());
  }
  void _pruneQueryFromTree(QueryRef query) {
    this._removeQueryRef(query);
    var child = this._head;
    while (isPresent(child)) {
      child._pruneQueryFromTree(query);
      child = child._next;
    }
  }
  void _addQueryToTree(QueryRef queryRef) {
    if (queryRef.query.descendants == false) {
      if (this == queryRef.originator) {
        this._addQueryToTreeSelfAndRecurse(queryRef);
      } else if (this.parent == queryRef.originator) {
        this._assignQueryRef(queryRef);
      }
    } else {
      this._addQueryToTreeSelfAndRecurse(queryRef);
    }
  }
  void _addQueryToTreeSelfAndRecurse(QueryRef queryRef) {
    this._assignQueryRef(queryRef);
    var child = this._head;
    while (isPresent(child)) {
      child._addQueryToTree(queryRef);
      child = child._next;
    }
  }
  void _assignQueryRef(QueryRef query) {
    if (isBlank(this._query0)) {
      this._query0 = query;
      return;
    } else if (isBlank(this._query1)) {
      this._query1 = query;
      return;
    } else if (isBlank(this._query2)) {
      this._query2 = query;
      return;
    }
    throw new QueryError();
  }
  void _removeQueryRef(QueryRef query) {
    if (this._query0 == query) this._query0 = null;
    if (this._query1 == query) this._query1 = null;
    if (this._query2 == query) this._query2 = null;
  }
  dynamic _getByKey(
      Key key, Visibility visibility, bool optional, Key requestor) {
    var ei = this;
    var currentVisibility = this._isComponentKey(requestor)
        ? LIGHT_DOM_AND_SHADOW_DOM
        :
        // and light dom dependencies
        LIGHT_DOM;
    var depth = visibility.depth;
    if (!visibility.includeSelf) {
      depth -= ei._proto.distanceToParent;
      if (isPresent(ei._parent)) {
        ei = ei._parent;
      } else {
        ei = ei._host;
        currentVisibility =
            visibility.crossComponentBoundaries ? LIGHT_DOM : SHADOW_DOM;
      }
    }
    while (ei != null && depth >= 0) {
      var preBuiltObj = ei._getPreBuiltObjectByKeyId(key.id);
      if (!identical(preBuiltObj, _undefined)) return preBuiltObj;
      var dir = ei._getObjByKeyId(key.id, currentVisibility);
      if (!identical(dir, _undefined)) return dir;
      depth -= ei._proto.distanceToParent;
      // we check only one mode with the SHADOW_DOM visibility
      if (identical(currentVisibility, SHADOW_DOM)) break;
      if (isPresent(ei._parent)) {
        ei = ei._parent;
      } else {
        ei = ei._host;
        currentVisibility =
            visibility.crossComponentBoundaries ? LIGHT_DOM : SHADOW_DOM;
      }
    }
    if (isPresent(this._host) && this._host._isComponentKey(key)) {
      return this._host.getComponent();
    } else if (optional) {
      return this._appInjector(requestor).getOptional(key);
    } else {
      return this._appInjector(requestor).get(key);
    }
  }
  Injector _appInjector(Key requestor) {
    if (isPresent(requestor) && this._isComponentKey(requestor)) {
      return this._shadowDomAppInjector;
    } else {
      return this._lightDomAppInjector;
    }
  }
  dynamic _getPreBuiltObjectByKeyId(num keyId) {
    var staticKeys = StaticKeys.instance();
    if (identical(keyId,
        staticKeys.viewManagerId)) return this._preBuiltObjects.viewManager;
    return _undefined;
  }
  _getObjByKeyId(num keyId, num visibility) {
    return this._strategy.getObjByKeyId(keyId, visibility);
  }
  getDirectiveAtIndex(num index) {
    return this._strategy.getDirectiveAtIndex(index);
  }
  bool hasInstances() {
    return this._constructionCounter > 0;
  }
  Injector getLightDomAppInjector() {
    return this._lightDomAppInjector;
  }
  Injector getShadowDomAppInjector() {
    return this._shadowDomAppInjector;
  }
  ElementInjector getHost() {
    return this._host;
  }
  num getBoundElementIndex() {
    return this._proto.index;
  }
}
abstract class _ElementInjectorStrategy {
  void callOnDestroy();
  void clearInstances();
  void hydrate();
  dynamic getComponent();
  bool isComponentKey(Key key);
  void buildQueries();
  void addDirectivesMatchingQuery(Query q, List<dynamic> res);
  dynamic getObjByKeyId(num keyId, num visibility);
  dynamic getDirectiveAtIndex(num index);
  DirectiveBinding getComponentBinding();
  num getMaxDirectives();
}
/**
 * Strategy used by the `ElementInjector` when the number of bindings is 10 or less.
 * In such a case, inlining fields is benefitial for performances.
 */
class ElementInjectorInlineStrategy implements _ElementInjectorStrategy {
  _ProtoElementInjectorInlineStrategy _protoStrategy;
  ElementInjector _ei;
  // If this element injector has a component, the component instance will be stored in _obj0
  dynamic _obj0 = null;
  dynamic _obj1 = null;
  dynamic _obj2 = null;
  dynamic _obj3 = null;
  dynamic _obj4 = null;
  dynamic _obj5 = null;
  dynamic _obj6 = null;
  dynamic _obj7 = null;
  dynamic _obj8 = null;
  dynamic _obj9 = null;
  ElementInjectorInlineStrategy(this._protoStrategy, this._ei) {}
  void callOnDestroy() {
    var p = this._protoStrategy;
    if (p._binding0 is DirectiveBinding &&
        ((p._binding0 as DirectiveBinding)).callOnDestroy) {
      this._obj0.onDestroy();
    }
    if (p._binding1 is DirectiveBinding &&
        ((p._binding1 as DirectiveBinding)).callOnDestroy) {
      this._obj1.onDestroy();
    }
    if (p._binding2 is DirectiveBinding &&
        ((p._binding2 as DirectiveBinding)).callOnDestroy) {
      this._obj2.onDestroy();
    }
    if (p._binding3 is DirectiveBinding &&
        ((p._binding3 as DirectiveBinding)).callOnDestroy) {
      this._obj3.onDestroy();
    }
    if (p._binding4 is DirectiveBinding &&
        ((p._binding4 as DirectiveBinding)).callOnDestroy) {
      this._obj4.onDestroy();
    }
    if (p._binding5 is DirectiveBinding &&
        ((p._binding5 as DirectiveBinding)).callOnDestroy) {
      this._obj5.onDestroy();
    }
    if (p._binding6 is DirectiveBinding &&
        ((p._binding6 as DirectiveBinding)).callOnDestroy) {
      this._obj6.onDestroy();
    }
    if (p._binding7 is DirectiveBinding &&
        ((p._binding7 as DirectiveBinding)).callOnDestroy) {
      this._obj7.onDestroy();
    }
    if (p._binding8 is DirectiveBinding &&
        ((p._binding8 as DirectiveBinding)).callOnDestroy) {
      this._obj8.onDestroy();
    }
    if (p._binding9 is DirectiveBinding &&
        ((p._binding9 as DirectiveBinding)).callOnDestroy) {
      this._obj9.onDestroy();
    }
  }
  void clearInstances() {
    this._obj0 = null;
    this._obj1 = null;
    this._obj2 = null;
    this._obj3 = null;
    this._obj4 = null;
    this._obj5 = null;
    this._obj6 = null;
    this._obj7 = null;
    this._obj8 = null;
    this._obj9 = null;
  }
  void hydrate() {
    var p = this._protoStrategy;
    var e = this._ei;
    if (isPresent(p._keyId0) && isBlank(this._obj0)) this._obj0 =
        e._new(p._binding0);
    if (isPresent(p._keyId1) && isBlank(this._obj1)) this._obj1 =
        e._new(p._binding1);
    if (isPresent(p._keyId2) && isBlank(this._obj2)) this._obj2 =
        e._new(p._binding2);
    if (isPresent(p._keyId3) && isBlank(this._obj3)) this._obj3 =
        e._new(p._binding3);
    if (isPresent(p._keyId4) && isBlank(this._obj4)) this._obj4 =
        e._new(p._binding4);
    if (isPresent(p._keyId5) && isBlank(this._obj5)) this._obj5 =
        e._new(p._binding5);
    if (isPresent(p._keyId6) && isBlank(this._obj6)) this._obj6 =
        e._new(p._binding6);
    if (isPresent(p._keyId7) && isBlank(this._obj7)) this._obj7 =
        e._new(p._binding7);
    if (isPresent(p._keyId8) && isBlank(this._obj8)) this._obj8 =
        e._new(p._binding8);
    if (isPresent(p._keyId9) && isBlank(this._obj9)) this._obj9 =
        e._new(p._binding9);
  }
  dynamic getComponent() {
    return this._obj0;
  }
  bool isComponentKey(Key key) {
    return this._ei._proto._firstBindingIsComponent &&
        isPresent(key) &&
        identical(key.id, this._protoStrategy._keyId0);
  }
  void buildQueries() {
    var p = this._protoStrategy;
    if (p._binding0 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding0.dependencies as List<DirectiveDependency>));
    }
    if (p._binding1 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding1.dependencies as List<DirectiveDependency>));
    }
    if (p._binding2 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding2.dependencies as List<DirectiveDependency>));
    }
    if (p._binding3 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding3.dependencies as List<DirectiveDependency>));
    }
    if (p._binding4 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding4.dependencies as List<DirectiveDependency>));
    }
    if (p._binding5 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding5.dependencies as List<DirectiveDependency>));
    }
    if (p._binding6 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding6.dependencies as List<DirectiveDependency>));
    }
    if (p._binding7 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding7.dependencies as List<DirectiveDependency>));
    }
    if (p._binding8 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding8.dependencies as List<DirectiveDependency>));
    }
    if (p._binding9 is DirectiveBinding) {
      this._ei._buildQueriesForDeps(
          (p._binding9.dependencies as List<DirectiveDependency>));
    }
  }
  void addDirectivesMatchingQuery(Query query, List<dynamic> list) {
    var p = this._protoStrategy;
    if (isPresent(p._binding0) &&
        identical(p._binding0.key.token, query.selector)) list.add(this._obj0);
    if (isPresent(p._binding1) &&
        identical(p._binding1.key.token, query.selector)) list.add(this._obj1);
    if (isPresent(p._binding2) &&
        identical(p._binding2.key.token, query.selector)) list.add(this._obj2);
    if (isPresent(p._binding3) &&
        identical(p._binding3.key.token, query.selector)) list.add(this._obj3);
    if (isPresent(p._binding4) &&
        identical(p._binding4.key.token, query.selector)) list.add(this._obj4);
    if (isPresent(p._binding5) &&
        identical(p._binding5.key.token, query.selector)) list.add(this._obj5);
    if (isPresent(p._binding6) &&
        identical(p._binding6.key.token, query.selector)) list.add(this._obj6);
    if (isPresent(p._binding7) &&
        identical(p._binding7.key.token, query.selector)) list.add(this._obj7);
    if (isPresent(p._binding8) &&
        identical(p._binding8.key.token, query.selector)) list.add(this._obj8);
    if (isPresent(p._binding9) &&
        identical(p._binding9.key.token, query.selector)) list.add(this._obj9);
  }
  dynamic getObjByKeyId(num keyId, num visibility) {
    var p = this._protoStrategy;
    if (identical(p._keyId0, keyId) && (p._visibility0 & visibility) > 0) {
      if (isBlank(this._obj0)) {
        this._obj0 = this._ei._new(p._binding0);
      }
      return this._obj0;
    }
    if (identical(p._keyId1, keyId) && (p._visibility1 & visibility) > 0) {
      if (isBlank(this._obj1)) {
        this._obj1 = this._ei._new(p._binding1);
      }
      return this._obj1;
    }
    if (identical(p._keyId2, keyId) && (p._visibility2 & visibility) > 0) {
      if (isBlank(this._obj2)) {
        this._obj2 = this._ei._new(p._binding2);
      }
      return this._obj2;
    }
    if (identical(p._keyId3, keyId) && (p._visibility3 & visibility) > 0) {
      if (isBlank(this._obj3)) {
        this._obj3 = this._ei._new(p._binding3);
      }
      return this._obj3;
    }
    if (identical(p._keyId4, keyId) && (p._visibility4 & visibility) > 0) {
      if (isBlank(this._obj4)) {
        this._obj4 = this._ei._new(p._binding4);
      }
      return this._obj4;
    }
    if (identical(p._keyId5, keyId) && (p._visibility5 & visibility) > 0) {
      if (isBlank(this._obj5)) {
        this._obj5 = this._ei._new(p._binding5);
      }
      return this._obj5;
    }
    if (identical(p._keyId6, keyId) && (p._visibility6 & visibility) > 0) {
      if (isBlank(this._obj6)) {
        this._obj6 = this._ei._new(p._binding6);
      }
      return this._obj6;
    }
    if (identical(p._keyId7, keyId) && (p._visibility7 & visibility) > 0) {
      if (isBlank(this._obj7)) {
        this._obj7 = this._ei._new(p._binding7);
      }
      return this._obj7;
    }
    if (identical(p._keyId8, keyId) && (p._visibility8 & visibility) > 0) {
      if (isBlank(this._obj8)) {
        this._obj8 = this._ei._new(p._binding8);
      }
      return this._obj8;
    }
    if (identical(p._keyId9, keyId) && (p._visibility9 & visibility) > 0) {
      if (isBlank(this._obj9)) {
        this._obj9 = this._ei._new(p._binding9);
      }
      return this._obj9;
    }
    return _undefined;
  }
  getDirectiveAtIndex(num index) {
    if (index == 0) return this._obj0;
    if (index == 1) return this._obj1;
    if (index == 2) return this._obj2;
    if (index == 3) return this._obj3;
    if (index == 4) return this._obj4;
    if (index == 5) return this._obj5;
    if (index == 6) return this._obj6;
    if (index == 7) return this._obj7;
    if (index == 8) return this._obj8;
    if (index == 9) return this._obj9;
    throw new OutOfBoundsAccess(index);
  }
  DirectiveBinding getComponentBinding() {
    return (this._protoStrategy._binding0 as DirectiveBinding);
  }
  num getMaxDirectives() {
    return _MAX_DIRECTIVE_CONSTRUCTION_COUNTER;
  }
}
/**
 * Strategy used by the `ElementInjector` when the number of bindings is 10 or less.
 * In such a case, inlining fields is benefitial for performances.
 */
class ElementInjectorDynamicStrategy implements _ElementInjectorStrategy {
  _ProtoElementInjectorDynamicStrategy _protoStrategy;
  ElementInjector _ei;
  // If this element injector has a component, the component instance will be stored in _objs[0]
  List<dynamic> _objs;
  ElementInjectorDynamicStrategy(this._protoStrategy, this._ei) {
    this._objs = ListWrapper.createFixedSize(_protoStrategy._bindings.length);
  }
  void callOnDestroy() {
    var p = this._protoStrategy;
    for (var i = 0; i < p._bindings.length; i++) {
      if (p._bindings[i] is DirectiveBinding &&
          ((p._bindings[i] as DirectiveBinding)).callOnDestroy) {
        this._objs[i].onDestroy();
      }
    }
  }
  void clearInstances() {
    ListWrapper.fill(this._objs, null);
  }
  void hydrate() {
    var p = this._protoStrategy;
    for (var i = 0; i < p._keyIds.length; i++) {
      if (isPresent(p._keyIds[i]) && isBlank(this._objs[i])) {
        this._objs[i] = this._ei._new(p._bindings[i]);
      }
    }
  }
  dynamic getComponent() {
    return this._objs[0];
  }
  bool isComponentKey(Key key) {
    return this._ei._proto._firstBindingIsComponent &&
        isPresent(key) &&
        identical(key.id, this._protoStrategy._keyIds[0]);
  }
  void buildQueries() {
    var p = this._protoStrategy;
    for (var i = 0; i < p._bindings.length; i++) {
      if (p._bindings[i] is DirectiveBinding) {
        this._ei._buildQueriesForDeps(
            (p._bindings[i].dependencies as List<DirectiveDependency>));
      }
    }
  }
  void addDirectivesMatchingQuery(Query query, List<dynamic> list) {
    var p = this._protoStrategy;
    for (var i = 0; i < p._bindings.length; i++) {
      if (identical(p._bindings[i].key.token, query.selector)) list
          .add(this._objs[i]);
    }
  }
  dynamic getObjByKeyId(num keyId, num visibility) {
    var p = this._protoStrategy;
    // TODO(vicb): optimize lookup ?
    for (var i = 0; i < p._keyIds.length; i++) {
      if (identical(p._keyIds[i], keyId) &&
          (p._visibilities[i] & visibility) > 0) {
        if (isBlank(this._objs[i])) {
          this._objs[i] = this._ei._new(p._bindings[i]);
        }
        return this._objs[i];
      }
    }
    return _undefined;
  }
  dynamic getDirectiveAtIndex(num index) {
    if (index < 0 || index >= this._objs.length) {
      throw new OutOfBoundsAccess(index);
    }
    return this._objs[index];
  }
  DirectiveBinding getComponentBinding() {
    return (this._protoStrategy._bindings[0] as DirectiveBinding);
  }
  num getMaxDirectives() {
    return this._objs.length;
  }
}
class OutOfBoundsAccess extends BaseException {
  String message;
  OutOfBoundsAccess(index) : super() {
    /* super call moved to initializer */;
    this.message = '''Index ${ index} is out-of-bounds.''';
  }
  String toString() {
    return this.message;
  }
}
class QueryError extends BaseException {
  String message;
  // TODO(rado): pass the names of the active directives.
  QueryError() : super() {
    /* super call moved to initializer */;
    this.message = "Only 3 queries can be concurrently active in a template.";
  }
  String toString() {
    return this.message;
  }
}
class QueryRef {
  Query query;
  QueryList<dynamic> list;
  ElementInjector originator;
  QueryRef(this.query, this.list, this.originator) {}
  void update() {
    var aggregator = [];
    this.visit(this.originator, aggregator);
    this.list.reset(aggregator);
  }
  void visit(ElementInjector inj, List<dynamic> aggregator) {
    if (isBlank(inj) || !inj._hasQuery(this)) return;
    if (this.query.isVarBindingQuery) {
      this._aggregateVariableBindings(inj, aggregator);
    } else {
      this._aggregateDirective(inj, aggregator);
    }
    var child = inj._head;
    while (isPresent(child)) {
      this.visit(child, aggregator);
      child = child._next;
    }
  }
  void _aggregateVariableBindings(
      ElementInjector inj, List<dynamic> aggregator) {
    var vb = this.query.varBindings;
    for (var i = 0; i < vb.length; ++i) {
      if (inj.hasVariableBinding(vb[i])) {
        aggregator.add(inj.getVariableBinding(vb[i]));
      }
    }
  }
  void _aggregateDirective(ElementInjector inj, List<dynamic> aggregator) {
    inj.addDirectivesMatchingQuery(this.query, aggregator);
  }
}
