library angular2.src.router.instruction;

import "package:angular2/src/facade/collection.dart"
    show Map, MapWrapper, Map, StringMapWrapper, List, ListWrapper;
import "package:angular2/src/facade/lang.dart" show isPresent, normalizeBlank;

class RouteParams {
  Map<String, String> params;
  RouteParams(this.params) {}
  String get(String param) {
    return normalizeBlank(StringMapWrapper.get(this.params, param));
  }
}
/**
 * An `Instruction` represents the component hierarchy of the application based on a given route
 */
class Instruction {
  dynamic component;
  Instruction child;
  // the part of the URL captured by this instruction
  String capturedUrl;
  // the part of the URL captured by this instruction and all children
  String accumulatedUrl;
  Map<String, String> params;
  bool reuse;
  num specificity;
  Instruction({params, component, child, matchedUrl, parentSpecificity}) {
    this.reuse = false;
    this.capturedUrl = matchedUrl;
    this.accumulatedUrl = matchedUrl;
    this.specificity = parentSpecificity;
    if (isPresent(child)) {
      this.child = child;
      this.specificity += child.specificity;
      var childUrl = child.accumulatedUrl;
      if (isPresent(childUrl)) {
        this.accumulatedUrl += childUrl;
      }
    } else {
      this.child = null;
    }
    this.component = component;
    this.params = params;
  }
  bool hasChild() {
    return isPresent(this.child);
  }
  /**
   * Takes a currently active instruction and sets a reuse flag on each of this instruction's
   * children
   */
  void reuseComponentsFrom(Instruction oldInstruction) {
    var nextInstruction = this;
    while (nextInstruction.reuse = shouldReuseComponent(
            nextInstruction, oldInstruction) &&
        isPresent(oldInstruction = oldInstruction.child) &&
        isPresent(nextInstruction = nextInstruction.child));
  }
}
bool shouldReuseComponent(Instruction instr1, Instruction instr2) {
  return instr1.component == instr2.component &&
      StringMapWrapper.equals(instr1.params, instr2.params);
}
